import { SchemaOf } from 'yup';
import { Path } from '../path';
import { State } from '../state';
import { FeatureBase, Token, TokenExtension } from '../token';
import { NonFunctionProperties } from '../utils/types';
import { ErrorMessage } from './error-message';
import { Schema } from './schema';
import { TouchedInfo } from './touched';
import { Validated } from './validation';
export interface FormState<TRead, TWrite = TRead> extends FeatureBase<'FormState', FormState<TRead, TWrite>> {
    payload: State<TRead, TWrite>['payload'] & Path['payload'] & ErrorMessage['payload'] & Validated['payload'] & Schema['payload'] & TouchedInfo['payload'];
    baseFeatures: State<TRead, TWrite> & Path & ErrorMessage & Validated & Schema & TouchedInfo;
    childFeatures: State<TRead, TWrite>['childFeatures'] & Path['childFeatures'] & ErrorMessage['childFeatures'] & Validated['childFeatures'] & Schema['childFeatures'] & TouchedInfo['childFeatures'] & {
        [P in keyof NonFunctionProperties<NonNullable<TRead>> & keyof NonFunctionProperties<NonNullable<TWrite>> & PropertyKey]-?: FormState<NonNullable<TRead>[P] | (TRead extends null | undefined ? undefined : never), NonNullable<TWrite>[P]>;
    };
}
export interface FormOptions<TState> {
    schema: SchemaOf<TState>;
    debounceValidationMs?: number;
}
export declare function addForm<TReadState, TWriteState>(options: FormOptions<TReadState>): TokenExtension<State<TReadState, TWriteState>, {
    add: FormState<TReadState, TWriteState>;
}>;
export declare function useFormToken<TState>(options: FormOptions<TState>): Token<FormState<TState>>;
