import { Path } from '../path';
import { State } from '../state';
import { FeatureBase, PartialToken, Token, TokenExtension } from '../token';
import { ErrorMessage, ErrorMessages } from './error-message';
import { TouchedInfo } from './touched';
export declare type ValidationStatusValues = 'pending' | 'validating' | 'invalid' | 'valid';
declare const _validationStatusToken: unique symbol;
export interface ValidationStatus extends FeatureBase<'ValidationStatus', ValidationStatus> {
    payload: {
        readonly [_validationStatusToken]: Token<State<ValidationStatusValues>>;
    };
    childFeatures: {
        [P in PropertyKey]: ValidationStatus;
    };
}
export interface Validated extends FeatureBase<'Validated', Validated> {
    payload: Path['payload'] & ErrorMessage['payload'] & TouchedInfo['payload'] & ValidationStatus['payload'] & {
        readonly [_validationStatusToken]: Token<State<ValidationStatusValues>>;
    };
    baseFeatures: Path & ErrorMessage & TouchedInfo & ValidationStatus;
    childFeatures: {
        [P in PropertyKey]: Validated;
    };
}
export declare type StateValidator<TState> = (value: TState | undefined) => Promise<ErrorMessages>;
export interface ValidationOptions<TState> {
    validator: StateValidator<TState>;
    debounceMs?: number;
}
export declare function addValidation<TReadState, TWriteState>(options: ValidationOptions<TReadState>): TokenExtension<State<TReadState, TWriteState>, {
    add: ErrorMessage & Validated;
}>;
export declare function getTokenValidationStatus(token: PartialToken<ValidationStatus>): ValidationStatusValues;
export declare function useTokenValidationStatus(token: PartialToken<ValidationStatus>): ValidationStatusValues;
export declare function useTokenValidationStatus(token: PartialToken<ValidationStatus> | undefined): ValidationStatusValues | undefined;
export {};
