'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');
var yup = require('yup');

var _path = /*#__PURE__*/Symbol('path');

function addPath(initialPath) {
  if (initialPath === void 0) {
    initialPath = '';
  }

  return function (tokenBuilder) {
    var _extend;

    return tokenBuilder.addFeature({
      extend: (_extend = {}, _extend[_path] = initialPath, _extend),
      extendChildren: function extendChildren(_, parent, property) {
        var _ref;

        var childPath;

        if (typeof property === 'number') {
          childPath = parent[_path] + "[" + String(property) + "]";
        } else {
          childPath = parent[_path] ? parent[_path] + "." + String(property) : String(property);
        }

        return _ref = {}, _ref[_path] = childPath, _ref;
      }
    });
  };
}
function getTokenPath(token) {
  return token[_path];
}

var _children = /*#__PURE__*/Symbol('children');
var _childrenExtension = /*#__PURE__*/Symbol('childrenExtension');
var _dispose = /*#__PURE__*/Symbol('dispose');

var _proxy = /*#__PURE__*/Symbol('proxy');

var _target = /*#__PURE__*/Symbol('target');

var positiveIntegerRegex = /^\d+$/;
var proxyHandler = {
  get: function get(target, property, receiver) {
    // Special property to retrieve proxy target
    if (property === _target) {
      return target;
    } // If target has the property, return it from target


    if (Reflect.has(target, property)) {
      return Reflect.get(target, property, receiver);
    } // For some reason proxies properties are string even when accessed with proxy[n];


    var convertedProperty = typeof property === 'string' && positiveIntegerRegex.test(property) ? parseInt(property) : property; // Reuse existing child token

    var childToken = target[_children][convertedProperty];

    if (!childToken) {
      // Create new child token
      childToken = new TokenBuilder().extend(function (builder) {
        return target[_childrenExtension](builder, target, convertedProperty);
      }).build();
      target[_children][convertedProperty] = childToken;
    }

    return childToken;
  }
};
function restoreToken(partialToken) {
  return toProxy(partialToken);
}

function toProxy(token) {
  if (token.hasOwnProperty(_proxy)) {
    return token[_proxy];
  }

  var proxy = new Proxy(token, proxyHandler);
  Object.defineProperty(token, _proxy, {
    value: proxy,
    configurable: false,
    writable: false
  });
  return proxy;
}

function getProxyTarget(tokenProxy) {
  return tokenProxy[_target];
}
function disposeToken(token) {
  token[_dispose]();
}
function useToken(extension) {
  var _useState = react.useState(function () {
    var builder = new TokenBuilder();

    if (extension) {
      builder = builder.extend(extension);
    }

    return builder.build();
  }),
      token = _useState[0];

  react.useEffect(function () {
    return function () {
      return disposeToken(token);
    };
  }, [token]);
  return token;
}
function useTokenExtension(token, extension) {
  var _useState2 = react.useState(function () {
    return new TokenBuilder(token).extend(extension).build();
  }),
      newToken = _useState2[0];

  react.useEffect(function () {
    return function () {
      return disposeToken(newToken);
    };
  }, [newToken]);
  return newToken;
}
var TokenBuilder = /*#__PURE__*/function () {
  function TokenBuilder(baseToken) {
    var _Object$assign2;

    this.partialToken = Object.create(baseToken ? getProxyTarget(baseToken) : Object.prototype);

    if (!baseToken) {
      var _Object$assign;

      Object.assign(this.partialToken, (_Object$assign = {}, _Object$assign[_childrenExtension] = function (token) {
        return token;
      }, _Object$assign));
    }

    Object.assign(this.partialToken, (_Object$assign2 = {}, _Object$assign2[_children] = {}, _Object$assign2[_dispose] = function () {}, _Object$assign2));
  }

  var _proto = TokenBuilder.prototype;

  _proto.replaceFeature = function replaceFeature(featureDescription) {
    return this.addFeature(featureDescription);
  };

  _proto.addFeature = function addFeature(featureDescription) {
    var extend = featureDescription.extend,
        extendChildren = featureDescription.extendChildren,
        dispose = featureDescription.dispose;
    this.partialToken = Object.create(this.partialToken);

    if (extend) {
      Object.assign(this.partialToken, extend);
    }

    if (extendChildren) {
      var interceptor = this.partialToken[_childrenExtension];

      if (typeof extendChildren === 'function') {
        this.partialToken[_childrenExtension] = function (childTokenBuilder, parentToken, property) {
          return interceptor(childTokenBuilder, parentToken, property).addFeature({
            extend: extendChildren(childTokenBuilder.partialToken, parentToken, property),
            extendChildren: extendChildren
          });
        };
      } else {
        this.partialToken[_childrenExtension] = function (childTokenBuilder, parent, property) {
          return interceptor(childTokenBuilder, parent, property).addFeature({
            extend: extend,
            extendChildren: extendChildren
          });
        };
      }
    }

    if (dispose) {
      var _Object$assign3;

      var previousDispose = this.partialToken[_dispose];
      Object.assign(this.partialToken, (_Object$assign3 = {}, _Object$assign3[_dispose] = function () {
        previousDispose();
        dispose();
      }, _Object$assign3));
    }

    return this;
  };

  _proto.extend = function extend(extension) {
    extension(this);
    return this;
  };

  _proto.peek = function peek() {
    return this.partialToken;
  };

  _proto.build = function build() {
    return restoreToken(this.partialToken);
  };

  return TokenBuilder;
}();

var _getValue = /*#__PURE__*/Symbol('getValue');

var _subscribe = /*#__PURE__*/Symbol('subscribe');

var _setValue = /*#__PURE__*/Symbol('setValue');

function addState(initialValue) {
  return function (builder) {
    var _extend;

    var value = initialValue;
    var subscriptions = new Map();
    var pathToken = builder.peek();
    return builder.addFeature({
      extend: (_extend = {}, _extend[_getValue] = function () {
        return value;
      }, _extend[_setValue] = function (newValue, path) {
        if (path === void 0) {
          path = getTokenPath(pathToken);
        }

        value = newValue;
        notifySubscribers(subscriptions, newValue, path);
      }, _extend[_subscribe] = function (listener) {
        return addSubscription(subscriptions, listener);
      }, _extend),
      extendChildren: function extendChildren(childToken, parent, property) {
        var _ref;

        return _ref = {}, _ref[_getValue] = function () {
          var parentValue = parent[_getValue]();

          return parentValue == null ? void 0 : parentValue[property];
        }, _ref[_setValue] = function (newValue, path) {
          var _Object$assign;

          if (path === void 0) {
            path = getTokenPath(childToken);
          }

          var base = typeof property === 'number' ? [] : {};

          parent[_setValue](Object.assign(base, parent[_getValue](), (_Object$assign = {}, _Object$assign[property] = newValue, _Object$assign)), path);
        }, _ref[_subscribe] = function (callback) {
          var snapshot = this[_getValue]();

          return parent[_subscribe](function (newParentValue, path) {
            var newValue = newParentValue == null ? void 0 : newParentValue[property];

            if (newValue !== snapshot) {
              snapshot = newValue;
              callback(newValue, path);
            }
          });
        }, _ref;
      }
    });
  };
}
function transformState(convertRead, convertWrite) {
  return function (builder) {
    var _extend2;

    var originalToken = builder.peek();
    var cachedValue = convertRead(originalToken[_getValue]());
    var subscriptions = new Map();
    var disposeSubscription = subscribeToTokenValue(originalToken, function (newOriginalValue, path) {
      var newValue = convertRead(newOriginalValue);

      if (newValue !== cachedValue) {
        cachedValue = newValue;
        notifySubscribers(subscriptions, newValue, path);
      }
    });
    return builder.replaceFeature({
      extend: (_extend2 = {}, _extend2[_getValue] = function () {
        return cachedValue;
      }, _extend2[_setValue] = function (newValue) {
        originalToken[_setValue](convertWrite(newValue));
      }, _extend2[_subscribe] = function (listener) {
        return addSubscription(subscriptions, listener);
      }, _extend2),
      dispose: disposeSubscription
    });
  };
}

function addSubscription(subscriptions, callback) {
  var _subscriptions$get;

  var unsubscribed = false;
  var count = (_subscriptions$get = subscriptions.get(callback)) != null ? _subscriptions$get : 0;
  subscriptions.set(callback, count + 1);
  return function () {
    var _subscriptions$get2;

    if (unsubscribed) return;
    unsubscribed = true;
    var count = (_subscriptions$get2 = subscriptions.get(callback)) != null ? _subscriptions$get2 : 0;

    if (count <= 1) {
      subscriptions["delete"](callback);
    } else {
      subscriptions.set(callback, count - 1);
    }
  };
}

function notifySubscribers(subscriptions, newValue, path) {
  Array.from(subscriptions.keys()).forEach(function (listener) {
    return listener(newValue, path);
  });
}

function getTokenValue(token) {
  return token[_getValue]();
}
function setTokenValue(token, newValue) {
  token[_setValue](newValue);
}
function subscribeToTokenValue(token, listener) {
  return token[_subscribe](listener);
} ////////////////
// React API
////////////////

function useStateToken(initializer) {
  return useToken(function (builder) {
    var initialValue = typeof initializer === 'function' ? initializer() : initializer;
    return builder.extend(addPath()).extend(addState(initialValue));
  });
}
function useTokenValue(token) {
  var _useState = react.useState(token ? getTokenValue(token) : undefined),
      value = _useState[0],
      setValue = _useState[1];

  react.useLayoutEffect(function () {
    if (!token) return;
    return subscribeToTokenValue(token, setValue);
  }, [token]);
  return value;
}
function useTokenSetter(token) {
  if (!token) return;
  return function (value) {
    return setTokenValue(token, value);
  };
}
function useTokenState(token) {
  return [useTokenValue(token), useTokenSetter(token)];
}

var _errorToken = /*#__PURE__*/Symbol('errorToken');

var _showAllErrors = /*#__PURE__*/Symbol('showAllErrors');

function addErrorMessages(options) {
  var errorsToken = options.errorsToken,
      showAllErrorsToken = options.showAllErrorsToken;
  return function (builder) {
    var _extend;

    var tokenBefore = builder.peek();
    return builder.addFeature({
      extend: (_extend = {}, _extend[_errorToken] = errorsToken[getTokenPath(tokenBefore)], _extend[_showAllErrors] = showAllErrorsToken, _extend),
      extendChildren: function extendChildren(childToken) {
        var _ref;

        return _ref = {}, _ref[_errorToken] = errorsToken[getTokenPath(childToken)], _ref[_showAllErrors] = showAllErrorsToken, _ref;
      }
    });
  };
}
function getTokenErrorMessage(token) {
  return getTokenValue(token[_errorToken]);
}
function getShowAllErrors(token) {
  return getTokenValue(token[_showAllErrors]);
}
function setShowAllErrors(token, value) {
  setTokenValue(token[_showAllErrors], value);
} ////////////////
// React API
////////////////

function useTokenErrorMessage(token) {
  return useTokenValue(token == null ? void 0 : token[_errorToken]);
}

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it;

  if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;
      return function () {
        if (i >= o.length) return {
          done: true
        };
        return {
          done: false,
          value: o[i++]
        };
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  it = o[Symbol.iterator]();
  return it.next.bind(it);
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var runtime_1 = createCommonjsModule(function (module) {
/**
 * Copyright (c) 2014-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

var runtime = (function (exports) {

  var Op = Object.prototype;
  var hasOwn = Op.hasOwnProperty;
  var undefined$1; // More compressible than void 0.
  var $Symbol = typeof Symbol === "function" ? Symbol : {};
  var iteratorSymbol = $Symbol.iterator || "@@iterator";
  var asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator";
  var toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";

  function define(obj, key, value) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
    return obj[key];
  }
  try {
    // IE 8 has a broken Object.defineProperty that only works on DOM objects.
    define({}, "");
  } catch (err) {
    define = function(obj, key, value) {
      return obj[key] = value;
    };
  }

  function wrap(innerFn, outerFn, self, tryLocsList) {
    // If outerFn provided and outerFn.prototype is a Generator, then outerFn.prototype instanceof Generator.
    var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator;
    var generator = Object.create(protoGenerator.prototype);
    var context = new Context(tryLocsList || []);

    // The ._invoke method unifies the implementations of the .next,
    // .throw, and .return methods.
    generator._invoke = makeInvokeMethod(innerFn, self, context);

    return generator;
  }
  exports.wrap = wrap;

  // Try/catch helper to minimize deoptimizations. Returns a completion
  // record like context.tryEntries[i].completion. This interface could
  // have been (and was previously) designed to take a closure to be
  // invoked without arguments, but in all the cases we care about we
  // already have an existing method we want to call, so there's no need
  // to create a new function object. We can even get away with assuming
  // the method takes exactly one argument, since that happens to be true
  // in every case, so we don't have to touch the arguments object. The
  // only additional allocation required is the completion record, which
  // has a stable shape and so hopefully should be cheap to allocate.
  function tryCatch(fn, obj, arg) {
    try {
      return { type: "normal", arg: fn.call(obj, arg) };
    } catch (err) {
      return { type: "throw", arg: err };
    }
  }

  var GenStateSuspendedStart = "suspendedStart";
  var GenStateSuspendedYield = "suspendedYield";
  var GenStateExecuting = "executing";
  var GenStateCompleted = "completed";

  // Returning this object from the innerFn has the same effect as
  // breaking out of the dispatch switch statement.
  var ContinueSentinel = {};

  // Dummy constructor functions that we use as the .constructor and
  // .constructor.prototype properties for functions that return Generator
  // objects. For full spec compliance, you may wish to configure your
  // minifier not to mangle the names of these two functions.
  function Generator() {}
  function GeneratorFunction() {}
  function GeneratorFunctionPrototype() {}

  // This is a polyfill for %IteratorPrototype% for environments that
  // don't natively support it.
  var IteratorPrototype = {};
  IteratorPrototype[iteratorSymbol] = function () {
    return this;
  };

  var getProto = Object.getPrototypeOf;
  var NativeIteratorPrototype = getProto && getProto(getProto(values([])));
  if (NativeIteratorPrototype &&
      NativeIteratorPrototype !== Op &&
      hasOwn.call(NativeIteratorPrototype, iteratorSymbol)) {
    // This environment has a native %IteratorPrototype%; use it instead
    // of the polyfill.
    IteratorPrototype = NativeIteratorPrototype;
  }

  var Gp = GeneratorFunctionPrototype.prototype =
    Generator.prototype = Object.create(IteratorPrototype);
  GeneratorFunction.prototype = Gp.constructor = GeneratorFunctionPrototype;
  GeneratorFunctionPrototype.constructor = GeneratorFunction;
  GeneratorFunction.displayName = define(
    GeneratorFunctionPrototype,
    toStringTagSymbol,
    "GeneratorFunction"
  );

  // Helper for defining the .next, .throw, and .return methods of the
  // Iterator interface in terms of a single ._invoke method.
  function defineIteratorMethods(prototype) {
    ["next", "throw", "return"].forEach(function(method) {
      define(prototype, method, function(arg) {
        return this._invoke(method, arg);
      });
    });
  }

  exports.isGeneratorFunction = function(genFun) {
    var ctor = typeof genFun === "function" && genFun.constructor;
    return ctor
      ? ctor === GeneratorFunction ||
        // For the native GeneratorFunction constructor, the best we can
        // do is to check its .name property.
        (ctor.displayName || ctor.name) === "GeneratorFunction"
      : false;
  };

  exports.mark = function(genFun) {
    if (Object.setPrototypeOf) {
      Object.setPrototypeOf(genFun, GeneratorFunctionPrototype);
    } else {
      genFun.__proto__ = GeneratorFunctionPrototype;
      define(genFun, toStringTagSymbol, "GeneratorFunction");
    }
    genFun.prototype = Object.create(Gp);
    return genFun;
  };

  // Within the body of any async function, `await x` is transformed to
  // `yield regeneratorRuntime.awrap(x)`, so that the runtime can test
  // `hasOwn.call(value, "__await")` to determine if the yielded value is
  // meant to be awaited.
  exports.awrap = function(arg) {
    return { __await: arg };
  };

  function AsyncIterator(generator, PromiseImpl) {
    function invoke(method, arg, resolve, reject) {
      var record = tryCatch(generator[method], generator, arg);
      if (record.type === "throw") {
        reject(record.arg);
      } else {
        var result = record.arg;
        var value = result.value;
        if (value &&
            typeof value === "object" &&
            hasOwn.call(value, "__await")) {
          return PromiseImpl.resolve(value.__await).then(function(value) {
            invoke("next", value, resolve, reject);
          }, function(err) {
            invoke("throw", err, resolve, reject);
          });
        }

        return PromiseImpl.resolve(value).then(function(unwrapped) {
          // When a yielded Promise is resolved, its final value becomes
          // the .value of the Promise<{value,done}> result for the
          // current iteration.
          result.value = unwrapped;
          resolve(result);
        }, function(error) {
          // If a rejected Promise was yielded, throw the rejection back
          // into the async generator function so it can be handled there.
          return invoke("throw", error, resolve, reject);
        });
      }
    }

    var previousPromise;

    function enqueue(method, arg) {
      function callInvokeWithMethodAndArg() {
        return new PromiseImpl(function(resolve, reject) {
          invoke(method, arg, resolve, reject);
        });
      }

      return previousPromise =
        // If enqueue has been called before, then we want to wait until
        // all previous Promises have been resolved before calling invoke,
        // so that results are always delivered in the correct order. If
        // enqueue has not been called before, then it is important to
        // call invoke immediately, without waiting on a callback to fire,
        // so that the async generator function has the opportunity to do
        // any necessary setup in a predictable way. This predictability
        // is why the Promise constructor synchronously invokes its
        // executor callback, and why async functions synchronously
        // execute code before the first await. Since we implement simple
        // async functions in terms of async generators, it is especially
        // important to get this right, even though it requires care.
        previousPromise ? previousPromise.then(
          callInvokeWithMethodAndArg,
          // Avoid propagating failures to Promises returned by later
          // invocations of the iterator.
          callInvokeWithMethodAndArg
        ) : callInvokeWithMethodAndArg();
    }

    // Define the unified helper method that is used to implement .next,
    // .throw, and .return (see defineIteratorMethods).
    this._invoke = enqueue;
  }

  defineIteratorMethods(AsyncIterator.prototype);
  AsyncIterator.prototype[asyncIteratorSymbol] = function () {
    return this;
  };
  exports.AsyncIterator = AsyncIterator;

  // Note that simple async functions are implemented on top of
  // AsyncIterator objects; they just return a Promise for the value of
  // the final result produced by the iterator.
  exports.async = function(innerFn, outerFn, self, tryLocsList, PromiseImpl) {
    if (PromiseImpl === void 0) PromiseImpl = Promise;

    var iter = new AsyncIterator(
      wrap(innerFn, outerFn, self, tryLocsList),
      PromiseImpl
    );

    return exports.isGeneratorFunction(outerFn)
      ? iter // If outerFn is a generator, return the full iterator.
      : iter.next().then(function(result) {
          return result.done ? result.value : iter.next();
        });
  };

  function makeInvokeMethod(innerFn, self, context) {
    var state = GenStateSuspendedStart;

    return function invoke(method, arg) {
      if (state === GenStateExecuting) {
        throw new Error("Generator is already running");
      }

      if (state === GenStateCompleted) {
        if (method === "throw") {
          throw arg;
        }

        // Be forgiving, per 25.3.3.3.3 of the spec:
        // https://people.mozilla.org/~jorendorff/es6-draft.html#sec-generatorresume
        return doneResult();
      }

      context.method = method;
      context.arg = arg;

      while (true) {
        var delegate = context.delegate;
        if (delegate) {
          var delegateResult = maybeInvokeDelegate(delegate, context);
          if (delegateResult) {
            if (delegateResult === ContinueSentinel) continue;
            return delegateResult;
          }
        }

        if (context.method === "next") {
          // Setting context._sent for legacy support of Babel's
          // function.sent implementation.
          context.sent = context._sent = context.arg;

        } else if (context.method === "throw") {
          if (state === GenStateSuspendedStart) {
            state = GenStateCompleted;
            throw context.arg;
          }

          context.dispatchException(context.arg);

        } else if (context.method === "return") {
          context.abrupt("return", context.arg);
        }

        state = GenStateExecuting;

        var record = tryCatch(innerFn, self, context);
        if (record.type === "normal") {
          // If an exception is thrown from innerFn, we leave state ===
          // GenStateExecuting and loop back for another invocation.
          state = context.done
            ? GenStateCompleted
            : GenStateSuspendedYield;

          if (record.arg === ContinueSentinel) {
            continue;
          }

          return {
            value: record.arg,
            done: context.done
          };

        } else if (record.type === "throw") {
          state = GenStateCompleted;
          // Dispatch the exception by looping back around to the
          // context.dispatchException(context.arg) call above.
          context.method = "throw";
          context.arg = record.arg;
        }
      }
    };
  }

  // Call delegate.iterator[context.method](context.arg) and handle the
  // result, either by returning a { value, done } result from the
  // delegate iterator, or by modifying context.method and context.arg,
  // setting context.delegate to null, and returning the ContinueSentinel.
  function maybeInvokeDelegate(delegate, context) {
    var method = delegate.iterator[context.method];
    if (method === undefined$1) {
      // A .throw or .return when the delegate iterator has no .throw
      // method always terminates the yield* loop.
      context.delegate = null;

      if (context.method === "throw") {
        // Note: ["return"] must be used for ES3 parsing compatibility.
        if (delegate.iterator["return"]) {
          // If the delegate iterator has a return method, give it a
          // chance to clean up.
          context.method = "return";
          context.arg = undefined$1;
          maybeInvokeDelegate(delegate, context);

          if (context.method === "throw") {
            // If maybeInvokeDelegate(context) changed context.method from
            // "return" to "throw", let that override the TypeError below.
            return ContinueSentinel;
          }
        }

        context.method = "throw";
        context.arg = new TypeError(
          "The iterator does not provide a 'throw' method");
      }

      return ContinueSentinel;
    }

    var record = tryCatch(method, delegate.iterator, context.arg);

    if (record.type === "throw") {
      context.method = "throw";
      context.arg = record.arg;
      context.delegate = null;
      return ContinueSentinel;
    }

    var info = record.arg;

    if (! info) {
      context.method = "throw";
      context.arg = new TypeError("iterator result is not an object");
      context.delegate = null;
      return ContinueSentinel;
    }

    if (info.done) {
      // Assign the result of the finished delegate to the temporary
      // variable specified by delegate.resultName (see delegateYield).
      context[delegate.resultName] = info.value;

      // Resume execution at the desired location (see delegateYield).
      context.next = delegate.nextLoc;

      // If context.method was "throw" but the delegate handled the
      // exception, let the outer generator proceed normally. If
      // context.method was "next", forget context.arg since it has been
      // "consumed" by the delegate iterator. If context.method was
      // "return", allow the original .return call to continue in the
      // outer generator.
      if (context.method !== "return") {
        context.method = "next";
        context.arg = undefined$1;
      }

    } else {
      // Re-yield the result returned by the delegate method.
      return info;
    }

    // The delegate iterator is finished, so forget it and continue with
    // the outer generator.
    context.delegate = null;
    return ContinueSentinel;
  }

  // Define Generator.prototype.{next,throw,return} in terms of the
  // unified ._invoke helper method.
  defineIteratorMethods(Gp);

  define(Gp, toStringTagSymbol, "Generator");

  // A Generator should always return itself as the iterator object when the
  // @@iterator function is called on it. Some browsers' implementations of the
  // iterator prototype chain incorrectly implement this, causing the Generator
  // object to not be returned from this call. This ensures that doesn't happen.
  // See https://github.com/facebook/regenerator/issues/274 for more details.
  Gp[iteratorSymbol] = function() {
    return this;
  };

  Gp.toString = function() {
    return "[object Generator]";
  };

  function pushTryEntry(locs) {
    var entry = { tryLoc: locs[0] };

    if (1 in locs) {
      entry.catchLoc = locs[1];
    }

    if (2 in locs) {
      entry.finallyLoc = locs[2];
      entry.afterLoc = locs[3];
    }

    this.tryEntries.push(entry);
  }

  function resetTryEntry(entry) {
    var record = entry.completion || {};
    record.type = "normal";
    delete record.arg;
    entry.completion = record;
  }

  function Context(tryLocsList) {
    // The root entry object (effectively a try statement without a catch
    // or a finally block) gives us a place to store values thrown from
    // locations where there is no enclosing try statement.
    this.tryEntries = [{ tryLoc: "root" }];
    tryLocsList.forEach(pushTryEntry, this);
    this.reset(true);
  }

  exports.keys = function(object) {
    var keys = [];
    for (var key in object) {
      keys.push(key);
    }
    keys.reverse();

    // Rather than returning an object with a next method, we keep
    // things simple and return the next function itself.
    return function next() {
      while (keys.length) {
        var key = keys.pop();
        if (key in object) {
          next.value = key;
          next.done = false;
          return next;
        }
      }

      // To avoid creating an additional object, we just hang the .value
      // and .done properties off the next function object itself. This
      // also ensures that the minifier will not anonymize the function.
      next.done = true;
      return next;
    };
  };

  function values(iterable) {
    if (iterable) {
      var iteratorMethod = iterable[iteratorSymbol];
      if (iteratorMethod) {
        return iteratorMethod.call(iterable);
      }

      if (typeof iterable.next === "function") {
        return iterable;
      }

      if (!isNaN(iterable.length)) {
        var i = -1, next = function next() {
          while (++i < iterable.length) {
            if (hasOwn.call(iterable, i)) {
              next.value = iterable[i];
              next.done = false;
              return next;
            }
          }

          next.value = undefined$1;
          next.done = true;

          return next;
        };

        return next.next = next;
      }
    }

    // Return an iterator with no values.
    return { next: doneResult };
  }
  exports.values = values;

  function doneResult() {
    return { value: undefined$1, done: true };
  }

  Context.prototype = {
    constructor: Context,

    reset: function(skipTempReset) {
      this.prev = 0;
      this.next = 0;
      // Resetting context._sent for legacy support of Babel's
      // function.sent implementation.
      this.sent = this._sent = undefined$1;
      this.done = false;
      this.delegate = null;

      this.method = "next";
      this.arg = undefined$1;

      this.tryEntries.forEach(resetTryEntry);

      if (!skipTempReset) {
        for (var name in this) {
          // Not sure about the optimal order of these conditions:
          if (name.charAt(0) === "t" &&
              hasOwn.call(this, name) &&
              !isNaN(+name.slice(1))) {
            this[name] = undefined$1;
          }
        }
      }
    },

    stop: function() {
      this.done = true;

      var rootEntry = this.tryEntries[0];
      var rootRecord = rootEntry.completion;
      if (rootRecord.type === "throw") {
        throw rootRecord.arg;
      }

      return this.rval;
    },

    dispatchException: function(exception) {
      if (this.done) {
        throw exception;
      }

      var context = this;
      function handle(loc, caught) {
        record.type = "throw";
        record.arg = exception;
        context.next = loc;

        if (caught) {
          // If the dispatched exception was caught by a catch block,
          // then let that catch block handle the exception normally.
          context.method = "next";
          context.arg = undefined$1;
        }

        return !! caught;
      }

      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        var record = entry.completion;

        if (entry.tryLoc === "root") {
          // Exception thrown outside of any try block that could handle
          // it, so set the completion value of the entire function to
          // throw the exception.
          return handle("end");
        }

        if (entry.tryLoc <= this.prev) {
          var hasCatch = hasOwn.call(entry, "catchLoc");
          var hasFinally = hasOwn.call(entry, "finallyLoc");

          if (hasCatch && hasFinally) {
            if (this.prev < entry.catchLoc) {
              return handle(entry.catchLoc, true);
            } else if (this.prev < entry.finallyLoc) {
              return handle(entry.finallyLoc);
            }

          } else if (hasCatch) {
            if (this.prev < entry.catchLoc) {
              return handle(entry.catchLoc, true);
            }

          } else if (hasFinally) {
            if (this.prev < entry.finallyLoc) {
              return handle(entry.finallyLoc);
            }

          } else {
            throw new Error("try statement without catch or finally");
          }
        }
      }
    },

    abrupt: function(type, arg) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.tryLoc <= this.prev &&
            hasOwn.call(entry, "finallyLoc") &&
            this.prev < entry.finallyLoc) {
          var finallyEntry = entry;
          break;
        }
      }

      if (finallyEntry &&
          (type === "break" ||
           type === "continue") &&
          finallyEntry.tryLoc <= arg &&
          arg <= finallyEntry.finallyLoc) {
        // Ignore the finally entry if control is not jumping to a
        // location outside the try/catch block.
        finallyEntry = null;
      }

      var record = finallyEntry ? finallyEntry.completion : {};
      record.type = type;
      record.arg = arg;

      if (finallyEntry) {
        this.method = "next";
        this.next = finallyEntry.finallyLoc;
        return ContinueSentinel;
      }

      return this.complete(record);
    },

    complete: function(record, afterLoc) {
      if (record.type === "throw") {
        throw record.arg;
      }

      if (record.type === "break" ||
          record.type === "continue") {
        this.next = record.arg;
      } else if (record.type === "return") {
        this.rval = this.arg = record.arg;
        this.method = "return";
        this.next = "end";
      } else if (record.type === "normal" && afterLoc) {
        this.next = afterLoc;
      }

      return ContinueSentinel;
    },

    finish: function(finallyLoc) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.finallyLoc === finallyLoc) {
          this.complete(entry.completion, entry.afterLoc);
          resetTryEntry(entry);
          return ContinueSentinel;
        }
      }
    },

    "catch": function(tryLoc) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.tryLoc === tryLoc) {
          var record = entry.completion;
          if (record.type === "throw") {
            var thrown = record.arg;
            resetTryEntry(entry);
          }
          return thrown;
        }
      }

      // The context.catch method must only be called with a location
      // argument that corresponds to a known catch block.
      throw new Error("illegal catch attempt");
    },

    delegateYield: function(iterable, resultName, nextLoc) {
      this.delegate = {
        iterator: values(iterable),
        resultName: resultName,
        nextLoc: nextLoc
      };

      if (this.method === "next") {
        // Deliberately forget the last sent value so that we don't
        // accidentally pass it on to the delegate.
        this.arg = undefined$1;
      }

      return ContinueSentinel;
    }
  };

  // Regardless of whether this script is executing as a CommonJS module
  // or not, return the runtime object so that we can declare the variable
  // regeneratorRuntime in the outer scope, which allows this module to be
  // injected easily by `bin/regenerator --include-runtime script.js`.
  return exports;

}(
  // If this script is executing as a CommonJS module, use module.exports
  // as the regeneratorRuntime namespace. Otherwise create a new empty
  // object. Either way, the resulting object will be used to initialize
  // the regeneratorRuntime variable at the top of this file.
   module.exports 
));

try {
  regeneratorRuntime = runtime;
} catch (accidentalStrictMode) {
  // This module should not be running in strict mode, so the above
  // assignment should always work unless something is misconfigured. Just
  // in case runtime.js accidentally runs in strict mode, we can escape
  // strict mode using a global Function call. This could conceivably fail
  // if a Content Security Policy forbids using Function, but in that case
  // the proper solution is to fix the accidental strict mode problem. If
  // you've misconfigured your bundler to force strict mode and applied a
  // CSP to forbid Function, and you're not willing to fix either of those
  // problems, please detail your unique predicament in a GitHub issue.
  Function("r", "regeneratorRuntime = r")(runtime);
}
});

var _schema = /*#__PURE__*/Symbol('schema');

function addSchema(schema) {
  return function (builder) {
    var _extend;

    return builder.addFeature({
      extend: (_extend = {}, _extend[_schema] = schema, _extend),
      extendChildren: function extendChildren(_, parent, property) {
        var _ref;

        var parentSchema = getTokenSchema(parent);
        var childSchema = parentSchema ? tryReach(parentSchema, String(property)) : undefined;
        return _ref = {}, _ref[_schema] = childSchema, _ref;
      }
    });
  };
}
function getTokenSchema(token) {
  return token[_schema];
}

function tryReach(obj, path) {
  try {
    return yup.reach(obj, path, null, null);
  } catch (_unused) {
    return null;
  }
} ////////////////
// React API
////////////////


function useTokenSchemaInfo(token) {
  var schemaDescription = react.useMemo(function () {
    if (!token) return;
    var schema = getTokenSchema(token);
    if (!schema) return;
    return schema.describe();
  }, [token]);
  if (!schemaDescription) return {};
  return {
    label: schemaDescription.label,
    required: Boolean(schemaDescription.tests.find(function (t) {
      return t.name === 'required';
    }))
  };
}

function debounce(fn, ms) {
  var timeoutId;
  return function () {
    clearTimeout(timeoutId);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    timeoutId = setTimeout(fn.bind.apply(fn, [this].concat(args)), ms);
  };
}

var _touchedToken = /*#__PURE__*/Symbol('touchedToken');

function addTouched(options) {
  var touchedFieldsToken = options.touchedFieldsToken;
  return function (builder) {
    var _extend;

    var tokenBefore = builder.peek();
    return builder.addFeature({
      extend: (_extend = {}, _extend[_touchedToken] = touchedFieldsToken[getTokenPath(tokenBefore)], _extend),
      extendChildren: function extendChildren(childToken) {
        var _ref;

        return _ref = {}, _ref[_touchedToken] = touchedFieldsToken[getTokenPath(childToken)], _ref;
      }
    });
  };
}
function getTokenTouched(token) {
  return getTokenValue(token[_touchedToken]);
}
function setTokenTouched(token, touched) {
  setTokenValue(token[_touchedToken], touched);
} ////////////////
// React API
////////////////

function useTokenTouched(token) {
  return Boolean(useTokenValue(token == null ? void 0 : token[_touchedToken]));
}

var _validationStatusToken = /*#__PURE__*/Symbol('validationStatusToken');

function addValidation(options) {
  var validator = options.validator,
      _options$debounceMs = options.debounceMs,
      debounceMs = _options$debounceMs === void 0 ? 200 : _options$debounceMs;
  return function (builder) {
    var _extend;

    var errorsToken = new TokenBuilder().extend(addPath()).extend(addState({})).build();
    var showAllErrorsToken = new TokenBuilder().extend(addPath()).extend(addState(false)).build();
    var validationStatusToken = new TokenBuilder().extend(addPath()).extend(addState('pending')).build();
    var touchedFieldsToken = new TokenBuilder().extend(addPath()).extend(addState({})).build();
    var lastStateVersion = 0;
    var pendingTouched = {};
    var debouncedValidate = debounce( /*#__PURE__*/function () {
      var _ref = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(newValue, version) {
        var errors;
        return runtime_1.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                setTokenValue(validationStatusToken, 'validating');
                _context.next = 3;
                return validator(newValue);

              case 3:
                errors = _context.sent;

                if (!(lastStateVersion !== version)) {
                  _context.next = 6;
                  break;
                }

                return _context.abrupt("return");

              case 6:
                {
                  console.info('Validation errors:', errors);
                }

                setTokenValue(errorsToken, errors);
                setTokenValue(validationStatusToken, Object.keys(errors).length === 0 ? 'valid' : 'invalid');
                setTokenValue(touchedFieldsToken, _extends({}, getTokenValue(touchedFieldsToken), pendingTouched));

              case 10:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }));

      return function (_x, _x2) {
        return _ref.apply(this, arguments);
      };
    }(), debounceMs);
    var tokenBefore = builder.peek();
    var disposeSubscription = subscribeToTokenValue(tokenBefore, /*#__PURE__*/function () {
      var _ref2 = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(newValue, path) {
        return runtime_1.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                setTokenValue(validationStatusToken, 'pending');
                lastStateVersion++;
                pendingTouched[path] = true;
                debouncedValidate(newValue, lastStateVersion);

              case 4:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2);
      }));

      return function (_x3, _x4) {
        return _ref2.apply(this, arguments);
      };
    }());
    debouncedValidate(getTokenValue(tokenBefore), lastStateVersion);
    return builder.extend(addErrorMessages({
      errorsToken: errorsToken,
      showAllErrorsToken: showAllErrorsToken
    })).extend(addTouched({
      touchedFieldsToken: touchedFieldsToken
    })).addFeature({
      extend: (_extend = {}, _extend[_validationStatusToken] = validationStatusToken, _extend),
      extendChildren: true,
      dispose: function dispose() {
        disposeSubscription();
      }
    });
  };
}
function getTokenValidationStatus(token) {
  return getTokenValue(token[_validationStatusToken]);
}
function useTokenValidationStatus(token) {
  return useTokenValue(token == null ? void 0 : token[_validationStatusToken]);
}

function addForm(options) {
  var schema = options.schema,
      _options$debounceVali = options.debounceValidationMs,
      debounceValidationMs = _options$debounceVali === void 0 ? 200 : _options$debounceVali;
  return function (builder) {
    return builder.extend(addSchema(schema)).extend(addValidation({
      validator: function () {
        var _validator = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(value) {
          return runtime_1.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  _context.next = 2;
                  return validateForm(schema, value);

                case 2:
                  return _context.abrupt("return", _context.sent);

                case 3:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee);
        }));

        function validator(_x) {
          return _validator.apply(this, arguments);
        }

        return validator;
      }(),
      debounceMs: debounceValidationMs
    }));
  };
}

function validateForm(_x2, _x3) {
  return _validateForm.apply(this, arguments);
} ////////////////
// React API
////////////////


function _validateForm() {
  _validateForm = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(schema, value) {
    var errors, _iterator, _step, error;

    return runtime_1.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            errors = {};
            _context2.prev = 1;
            _context2.next = 4;
            return schema.validate(value, {
              strict: true,
              abortEarly: false
            });

          case 4:
            _context2.next = 17;
            break;

          case 6:
            _context2.prev = 6;
            _context2.t0 = _context2["catch"](1);

            if (!yup.ValidationError.isError(_context2.t0)) {
              _context2.next = 17;
              break;
            }

            _iterator = _createForOfIteratorHelperLoose(_context2.t0.inner);

          case 10:
            if ((_step = _iterator()).done) {
              _context2.next = 17;
              break;
            }

            error = _step.value;

            if (error.path) {
              _context2.next = 14;
              break;
            }

            return _context2.abrupt("continue", 15);

          case 14:
            if (errors[error.path]) {
              errors[error.path] += ' ' + error.message;
            } else {
              errors[error.path] = error.message;
            }

          case 15:
            _context2.next = 10;
            break;

          case 17:
            return _context2.abrupt("return", errors);

          case 18:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, null, [[1, 6]]);
  }));
  return _validateForm.apply(this, arguments);
}

function useFormToken(options) {
  var schema = options.schema;
  var valueToken = useStateToken(function () {
    return schema.getDefault();
  });
  return useTokenExtension(valueToken, function (builder) {
    return builder.extend(addForm(options));
  });
}

exports.TokenBuilder = TokenBuilder;
exports._children = _children;
exports._childrenExtension = _childrenExtension;
exports._dispose = _dispose;
exports.addErrorMessages = addErrorMessages;
exports.addForm = addForm;
exports.addPath = addPath;
exports.addSchema = addSchema;
exports.addState = addState;
exports.addTouched = addTouched;
exports.addValidation = addValidation;
exports.disposeToken = disposeToken;
exports.getProxyTarget = getProxyTarget;
exports.getShowAllErrors = getShowAllErrors;
exports.getTokenErrorMessage = getTokenErrorMessage;
exports.getTokenPath = getTokenPath;
exports.getTokenSchema = getTokenSchema;
exports.getTokenTouched = getTokenTouched;
exports.getTokenValidationStatus = getTokenValidationStatus;
exports.getTokenValue = getTokenValue;
exports.restoreToken = restoreToken;
exports.setShowAllErrors = setShowAllErrors;
exports.setTokenTouched = setTokenTouched;
exports.setTokenValue = setTokenValue;
exports.subscribeToTokenValue = subscribeToTokenValue;
exports.transformState = transformState;
exports.useFormToken = useFormToken;
exports.useStateToken = useStateToken;
exports.useToken = useToken;
exports.useTokenErrorMessage = useTokenErrorMessage;
exports.useTokenExtension = useTokenExtension;
exports.useTokenSchemaInfo = useTokenSchemaInfo;
exports.useTokenSetter = useTokenSetter;
exports.useTokenState = useTokenState;
exports.useTokenTouched = useTokenTouched;
exports.useTokenValidationStatus = useTokenValidationStatus;
exports.useTokenValue = useTokenValue;
//# sourceMappingURL=react-use-token.cjs.development.js.map
