import { Dispatch } from 'react';
import { Path } from '../path';
import { FeatureBase, PartialToken, Token, TokenExtension } from '../token';
import { NonFunctionProperties } from '../utils/types';
declare const _getValue: unique symbol;
declare const _subscribe: unique symbol;
declare const _setValue: unique symbol;
export interface State<TRead, TWrite = TRead> extends FeatureBase<'state', State<TRead, TWrite>> {
    payload: Path['payload'] & {
        [_getValue](): TRead;
        [_subscribe](listener: TokenValueListener<TRead>): Disposer;
        [_setValue](newValue: TWrite, path?: string): void;
    };
    baseFeatures: Path;
    childFeatures: Path['childFeatures'] & {
        [P in keyof NonFunctionProperties<NonNullable<TRead>>]-?: State<NonNullable<TRead>[P] | (TRead extends null | undefined ? undefined : never), NonNullable<TRead>[P]>;
    };
}
export declare type TokenValueListener<TState> = (newValue: TState, path: string) => void;
export declare type Disposer = () => void;
export declare function addState<TState>(initialValue: TState): TokenExtension<Path, {
    add: State<TState>;
}>;
export declare function transformState<TOldReadState, TNewReadState, TOldWriteState = TOldReadState, TNewWriteState = TNewReadState>(convertRead: (value: TOldReadState) => TNewReadState, convertWrite: (value: TNewWriteState) => TOldWriteState): TokenExtension<State<TOldReadState, TOldWriteState>, {
    remove: State<TOldReadState, TOldWriteState>;
    add: State<TNewReadState, TNewWriteState>;
}>;
export declare function getTokenValue<TState>(token: PartialToken<State<TState, unknown>>): TState;
export declare function setTokenValue<TState>(token: PartialToken<State<unknown, TState>>, newValue: TState): void;
export declare function subscribeToTokenValue<TState>(token: PartialToken<State<TState, unknown>>, listener: TokenValueListener<TState>): Disposer;
export declare function useStateToken<T>(initializer: T | (() => T)): Token<State<T>>;
export declare function useTokenValue<TState>(token: PartialToken<State<TState, unknown>>): TState;
export declare function useTokenValue<TState>(token: PartialToken<State<TState, unknown>> | undefined): TState | undefined;
export declare function useTokenSetter<TState>(token: PartialToken<State<unknown, TState>>): Dispatch<TState>;
export declare function useTokenSetter<TState>(token: PartialToken<State<unknown, TState>> | undefined): Dispatch<TState> | undefined;
export declare function useTokenState<TReadState, TWriteState = TReadState>(token: PartialToken<State<TReadState, TWriteState>>): [TReadState, Dispatch<TWriteState>];
export declare function useTokenState<TReadState, TWriteState = TReadState>(token: PartialToken<State<TReadState, TWriteState>> | undefined): [TReadState | undefined, Dispatch<TWriteState> | undefined];
export {};
