import { Cast, IfNever, IfUnknown, Remap, UnionToIntersection } from '../utils/types';
export declare type NoFeature = never;
export interface AnyFeature {
    self: {
        [U in string]: AnyFeature;
    };
    payload?: Record<symbol, any>;
    baseFeatures?: AnyFeature;
    childFeatures?: Record<PropertyKey, AnyFeature>;
}
export interface FeatureBase<TId extends string, TSelf> {
    self: {
        [U in TId]: TSelf;
    };
}
export declare type FeatureId<Features extends AnyFeature> = keyof IfNever<Features['self'], {}>;
export declare type FeaturesPayload<Features extends AnyFeature> = Features['payload'];
export declare type BaseFeatures<Features extends AnyFeature> = NonNullable<Features['baseFeatures']>;
export declare type ChildFeatures<Features extends AnyFeature> = Cast<Remap<NonNullable<Features['childFeatures']>>, Record<PropertyKey, AnyFeature>>;
export declare type ExcludeFeatures<Features extends AnyFeature, FeaturesToExclude extends AnyFeature> = Cast<UnionToIntersection<Features['self'][Cast<Exclude<FeatureId<Features>, FeatureId<FeaturesToExclude>>, keyof Features['self']>]>, AnyFeature>;
export declare type CleanupFeatures<Features extends AnyFeature> = ExcludeFeatures<Features, BaseFeatures<Features>>;
export interface FeaturesTransformation {
    remove?: AnyFeature;
    add?: AnyFeature;
}
export declare type TransformFeatures<Features extends AnyFeature, Transformation extends FeaturesTransformation> = Cast<IfUnknown<IfNever<ExcludeFeatures<Features, IfUnknown<NonNullable<Transformation['remove']>, never>>, unknown> & Transformation['add'], never>, AnyFeature>;
