import { IfNever } from '../utils/types';
import { AnyFeature, ChildFeatures, CleanupFeatures, FeaturesPayload, FeaturesTransformation, NoFeature, TransformFeatures } from './feature';
export declare const _children: unique symbol;
export declare const _childrenExtension: unique symbol;
export declare const _dispose: unique symbol;
export interface TokenBase {
    [_children]: Record<PropertyKey, any>;
    [_childrenExtension]: ChildTokenExtension<NoFeature, any>;
    [_dispose]: () => void;
}
export declare type PartialToken<Features extends AnyFeature> = TokenBase & IfNever<Features, unknown, FeaturesPayload<Features>>;
export declare type Token<Features extends AnyFeature> = PartialToken<Features> & ChildTokensProperties<Features>;
export declare type ChildTokensProperties<Features extends AnyFeature> = {
    readonly [k in keyof ChildFeatures<Features>]: Token<CleanupFeatures<ChildFeatures<Features>[k]>>;
};
export declare function restoreToken<Features extends AnyFeature>(partialToken: PartialToken<Features>): Token<Features>;
export declare function getProxyTarget<Features extends AnyFeature>(tokenProxy: PartialToken<Features>): PartialToken<Features>;
export interface TokenExtension<Features extends AnyFeature, Transformations extends FeaturesTransformation = {}> {
    (builder: TokenBuilder<Features>): TokenBuilder<any, Transformations>;
}
interface ChildTokenExtension<FeaturesBefore extends AnyFeature, FeaturesAfter extends AnyFeature> {
    (token: TokenBuilder<FeaturesBefore>, parentToken: Token<FeaturesAfter>, property: PropertyKey): TokenBuilder<FeaturesAfter>;
}
export declare function disposeToken(token: PartialToken<never>): void;
export declare function useToken(): Token<NoFeature>;
export declare function useToken<Transformations extends FeaturesTransformation>(extension: TokenExtension<NoFeature, Transformations>): Token<TransformFeatures<NoFeature, Transformations>>;
export declare function useTokenExtension<Features extends BaseFeatures, BaseFeatures extends AnyFeature = Features, Transformations extends FeaturesTransformation = {}>(token: Token<Features>, extension: TokenExtension<BaseFeatures, Transformations>): Token<TransformFeatures<Features, Transformations>>;
export declare class TokenBuilder<Features extends AnyFeature = NoFeature, Transformations extends FeaturesTransformation = {}> {
    private partialToken;
    constructor(baseToken?: Token<Features>);
    replaceFeature<OldFeatures extends AnyFeature, NewFeatures extends AnyFeature>(featureDescription: FeatureDescription<Features, NewFeatures>): TokenBuilder<TransformFeatures<Features, {
        remove: OldFeatures;
        add: NewFeatures;
    }>, Transformations & {
        remove: OldFeatures;
        add: NewFeatures;
    }>;
    addFeature<NewFeatures extends AnyFeature>(featureDescription: FeatureDescription<Features, NewFeatures>): TokenBuilder<TransformFeatures<Features, {
        add: NewFeatures;
    }>, Transformations & {
        add: NewFeatures;
    }>;
    extend<BaseFeatures extends AnyFeature = Features, NewTransformations extends FeaturesTransformation = {}>(extension: TokenExtension<BaseFeatures, NewTransformations>): TokenBuilder<TransformFeatures<Features, NewTransformations>, Transformations & NewTransformations>;
    peek(): PartialToken<Features>;
    build(): Token<Features>;
}
export interface FeatureDescription<BaseFeatures extends AnyFeature, NewFeature extends AnyFeature> {
    extend: Omit<NewFeature['payload'], keyof BaseFeatures['payload']>;
    extendChildren?: boolean | ((childToken: PartialToken<BaseFeatures>, parentToken: Token<NewFeature>, property: PropertyKey) => Omit<NewFeature['payload'], keyof BaseFeatures['payload']>);
    dispose?: () => void;
}
export {};
